#!/usr/bin/env python
# coding: utf-8

# In[1]:


import random

class Ballot():
    def __init__(self, num_choices):
        # Pick random choices.
        self.choices = list(range(num_choices))
        random.shuffle(self.choices)

    def __str__(self):
        return f'{self.choices}'

    def top_choice(self, eliminated):
        '''Find this ballot's top non-disqualified choice.'''
        for candidate in self.choices:
            if not eliminated[candidate]: return candidate
        return None


# In[2]:


def make_ballots(num_ballots, num_choices):
    '''Create some random ballots.'''
    return [Ballot(num_choices) for i in range(num_ballots)]


# In[3]:


def show_counts(ballots, eliminated):
    '''Count the number of votes for each choice and return the counts.'''
    # Count each ballot's top choice.
    num_choices = len(ballots[0].choices)
    counts = [0 for i in range(num_choices)]
    for ballot in ballots:
        top_choice = ballot.top_choice(eliminated)
        if top_choice is not None:
            counts[top_choice] += 1

    # Display the counts.
    for i in range(num_choices):
        print(counts[i], end=' ')
    print()

    return counts


# In[4]:


import math

def simulate_votes(num_ballots, num_choices):
    # Make random ballots.
    needed_to_win = math.floor(num_ballots / 2 + 1)
    print(f'Needed to win: {needed_to_win}')
    ballots = make_ballots(num_ballots, num_choices)
    
    # Initially no choice is eliminated.
    eliminated = [False for i in range(num_choices)]
    
    # Run through the rounds.
    winner = None
    for i in range(num_choices):
        # Display the current counts.
        print(f'Round {i+1}')
        counts = show_counts(ballots, eliminated)
    
        # See if there is a winner.
        for j in range(num_choices):
            if counts[j] >= needed_to_win:
                winner = j
                break
        if winner is not None: break
    
        # There was no winner. Eliminate the least popular choice.
        # Find the smallest vote count.
        eligible_counts = [counts[k] for k in range(num_choices) if not eliminated[k]]
        if len(eligible_counts) == 0:
            print('All choices have been eliminated')
            break
        smallest = min(eligible_counts)
        for k in range(num_choices):
            if counts[k] == smallest:
                eliminated[k] = True
                print(f'Eliminating {k} which has {counts[k]} votes')
        print()
    
    if winner is not None:
        print(f'The winner is {winner} with {counts[winner]} votes')


# In[5]:


simulate_votes(100, 5)


# In[ ]:




